@@ -1,7 +1,9 @@
+#!/bin/sh
+
 cp ../../pngminus/png2pnm.c pngm2pnm.c
 cp ../../../*.h .
 cp ../../../*.c .
-rm example.c pngtest.c pngpread.c pngw*.c
+rm -f example.c pngtest.c pngpread.c pngw*.c pnggccrd.c pngvcrd.c
 # change the following 2 lines if zlib is somewhere else
 cp ../../../../zlib/*.h .
 cp ../../../../zlib/*.c .
